// This file is a fixture for naming convention E2E tests.
// It contains various naming patterns that may or may not violate name_deny rules.

pub struct UserService;

pub fn execute() {}

pub const MAX_RETRIES: u32 = 3;
